/*
Copyright 2016 Ian Jauslin

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
  Mutable arrays
*/

#ifndef LIBINUM_ARRAY_H
#define LIBINUM_ARRAY_H

#include "types.h"

//--------------------------------------------------
//
//  integer array (array_int)
//
//--------------------------------------------------
#include "array_int.h"
#include "array_base.h"


//--------------------------------------------------
//
//  unsigned integer array (array_uint)
//
//--------------------------------------------------
#include "array_uint.h"
#include "array_base.h"


//--------------------------------------------------
//
//  double array (array_double)
//
//--------------------------------------------------
#include "array_double.h"
#include "array_base.h"


//--------------------------------------------------
//
//  long double array (array_ldouble)
//
//--------------------------------------------------
#include "array_ldouble.h"
#include "array_base.h"


//--------------------------------------------------
//
//  MPFR array (array_mpfr)
//
//--------------------------------------------------
#include "array_mpfr.h"
#include "array_base.h"


//--------------------------------------------------
//
//  array of MPFR arrays (array_2_mpfr)
//
//--------------------------------------------------
#include "array_2_mpfr.h"
#include "array_base.h"


//--------------------------------------------------
//
//  character array (array_char)
//
//--------------------------------------------------
#include "array_char.h"
#include "array_base.h"

// append a char*
int array_char_append_str(char* str, array_char* output);

// convert to char*
int array_char_to_str(array_char input, char** output);
// noinit (changes the size of input if needed)
char* array_char_to_str_noinit(array_char* input);

// convert from char*
int str_to_array_char(char* str, array_char* output);

// compare an array_char and a char*
int array_char_cmp_str(array_char array_char, char* str);

// format strings
int array_char_snprintf(array_char* output, char* fmt, ...);


//--------------------------------------------------
//
//  string array (array_str)
//
//--------------------------------------------------
#include "array_str.h"
#include "array_base.h"


//--------------------------------------------------
//
//  mpfr polynomial array (array_polynomial_mpfr)
//
//--------------------------------------------------
#include "array_polynomial_mpfr.h"
#include "array_base.h"


//--------------------------------------------------
//
//  double polynomial array (array_polynomial_double)
//
//--------------------------------------------------
#include "array_polynomial_double.h"
#include "array_base.h"


//--------------------------------------------------
//
//  long double polynomial array (array_polynomial_ldouble)
//
//--------------------------------------------------
#include "array_polynomial_ldouble.h"
#include "array_base.h"


//--------------------------------------------------
//
//  pthread_t array (array_pthread_t)
//
//--------------------------------------------------
#include "array_pthread_t.h"
#include "array_base.h"

#endif
