/*
Copyright 2016 Ian Jauslin

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
  Preprocessor macros for arrays of MPFR float arrays
*/


// reset CPP macros
#undef ARRAY_TYPENAME
#undef ARRAY_FUNC
#undef ARRAY_VAL_TYPE
#undef ARRAY_VAL_INIT
#undef ARRAY_VAL_FREE
#undef ARRAY_VAL_SET
#undef ARRAY_VAL_CPY
#undef ARRAY_VAL_IFEQ
#undef ARRAY_VAL_IFLT
#undef ARRAY_VAL_IFGT
#undef ARRAY_VAL_PRINT


// name of the polynomial type
#define ARRAY_TYPENAME array_2_mpfr
// prefix of function names
#define ARRAY_FUNC(NAME) array_2_mpfr_ ## NAME

// type of values
#define ARRAY_VAL_TYPE array_mpfr
// init
#define ARRAY_VAL_INIT(VAL, SIZE) array_mpfr_init(VAL, SIZE)
// free
#define ARRAY_VAL_FREE(VAL) array_mpfr_free(VAL)
// set values
#define ARRAY_VAL_SET(VAL, NEWVAL) VAL=NEWVAL
// copy values
#define ARRAY_VAL_CPY(FROMVAL, TOVAL) array_mpfr_cpy(FROMVAL, &TOVAL);
// compare values
#define ARRAY_VAL_IFEQ(VAL1, VAL2) array_mpfr_cmp(VAL1, VAL2)==0
#define ARRAY_VAL_IFLT(VAL1, VAL2) array_mpfr_cmp(VAL1, VAL2)<0
#define ARRAY_VAL_IFGT(VAL1, VAL2) array_mpfr_cmp(VAL1, VAL2)>0
// print values
#define ARRAY_VAL_PRINT(VAL) array_mpfr_print(VAL)

